/*
//	Description: Demo SPI Functions
//	File:		SPITest.cpp
//	Created:	Christian Apfel
//	Date:		01.04.2016
//	Modified:	Christian Apfel
//	Date:		10.08.2016
//
*/

#include "stdafx.h"
#include "SPITest.h"

using namespace std;

SPITest::SPITest()
{

}

SPITest::~SPITest()
{

}

// ****** Demo App Menu Functions ******

void SPITest::StartSpiTest()
{
	int SPI_TEST;
	int SPI_DEV;
	int SPI_DEV_MODE;
	int SPI_REG_ADDR;
	int SPI_MSG_LEN;
	int SPI_DEV_LOOPBACK;
	unsigned int SPI_DATA;
	unsigned int SPI_REVERSE_DATA;
	unsigned int *SPI_DATA_ARRAY;
	char *cStatus = new char[20];
	bool bLoopbackEnable = false;

	// Wait Loop for 0 to Exit
	do
	{
		bLoopbackEnable = false;

		cout << "---------------------------------" << endl;
		cout << "Select SPI Test!" << endl;
		cout << "---------------------------------" << endl;
		cout << "1: Write Test" << endl;
		cout << "2: Read Test" << endl;
		cout << "3: Write Multiple Test" << endl;
		cout << "4: Read Multiple Test" << endl;
		cout << "5: Write NOR Flash Test" << endl;
		cout << "6: Read NOR Flash Test" << endl;
		cout << "0: Main Menu" << endl;
		cout << "---------------------------------" << endl;
		cout << "Enter Test Number: ";
		cin >> SPI_TEST;

		switch(SPI_TEST)
		{
		case 1:
			{
				cout << endl;
				cout << "SPI Write (Loopback) Test" << endl;
				cout << "------------------------------------" << endl;
				cout << "Enter SPI Device (1,2): ";
				cin >> SPI_DEV;
				cout << "Enter SPI Mode (0..3): ";
				cin >> SPI_DEV_MODE;
				cout << "Enter SPI Data (e.g. 1 = 0x01): ";
				cin >> std::hex >> SPI_DATA;
				cout << "Enter Loopback Mode (0=Disable, 1=Enable): ";
				cin >> SPI_DEV_LOOPBACK;

				if(SPI_DEV_LOOPBACK == 1)
				{
					bLoopbackEnable = true;
				}

				if(SPIWriteTest(DHCOM_SPIEnum(SPI_DEV - 1), DHCOM_SPIModeEnum(SPI_DEV_MODE), &SPI_DATA, &SPI_REVERSE_DATA, bLoopbackEnable) == true)
				{
					cStatus = "Success";
				}
				else
				{
					cStatus = "Failed";
				}

				if(SPI_DEV_LOOPBACK == 1)
				{
					cout << "SPI Write Test - Data: " << SPI_REVERSE_DATA << " Status: " << cStatus << "!!!" << endl;
				}
				else
				{
					cout << "SPI Write Test (Loopback) - Reverse Data: " << SPI_DATA << " Status: " << cStatus << "!!!" << endl;
				}

				cout << "------------------------------------" << endl;
			}
			break;
		case 2:
			{
				cout << endl;
				cout << "SPI Read Test" << endl;
				cout << "------------------------------------" << endl;
				cout << "Enter SPI Device (1,2): ";
				cin >> SPI_DEV;
				cout << "Enter SPI Mode (0..3): ";
				cin >> SPI_DEV_MODE;

				SPI_DATA = 0;

				if(SPIReadTest(DHCOM_SPIEnum(SPI_DEV - 1), DHCOM_SPIModeEnum(SPI_DEV_MODE), &SPI_DATA) == true)
				{
					cStatus = "Success";
				}
				else
				{
					cStatus = "Failed";
				}

				cout << "SPI Read Test from SPI with Data " << SPI_DATA << " is " << cStatus << "!!!" << endl;
				cout << "------------------------------------" << endl;
			}
			break;
		case 3:
			{
				cout << endl;
				cout << "SPI Write Multiple Test" << endl;
				cout << "------------------------------------" << endl;
				cout << "Enter SPI Device (1,2): ";
				cin >> SPI_DEV;
				cout << "Enter SPI Mode (0..3): ";
				cin >> SPI_DEV_MODE;
				cout << "Enter SPI Data Len (e.g. 1): ";
				cin >> SPI_MSG_LEN;

				// Allocate Data Array
				SPI_DATA_ARRAY = (unsigned int*) malloc(SPI_MSG_LEN * sizeof(unsigned int));

				for(int i = 0; i < SPI_MSG_LEN; i++)
				{
					cout << "Enter SPI Data[" << i << "] (e.g. 0xFFFF): ";
					cin >> std::hex >> SPI_DATA_ARRAY[i];
				}

				if(SPIWriteMultipleTest(DHCOM_SPIEnum(SPI_DEV - 1), DHCOM_SPIModeEnum(SPI_DEV_MODE), SPI_MSG_LEN, SPI_DATA_ARRAY) == true)
				{
					cStatus = "Success";
				}
				else
				{
					cStatus = "Failed";
				}

				cout << "SPI Write Multiple Test - Data: " << SPI_DATA_ARRAY << " is " << cStatus << "!!!" << endl;
				cout << "------------------------------------" << endl;

				delete(SPI_DATA_ARRAY);
			}
			break;
		case 4:
			{
				cout << endl;
				cout << "SPI Read Multiple Test" << endl;
				cout << "------------------------------------" << endl;
				cout << "Enter SPI Device (1,2): ";
				cin >> SPI_DEV;
				cout << "Enter SPI Mode (0..3): ";
				cin >> SPI_DEV_MODE;
				cout << "Enter SPI Data Len (e.g. 1): ";
				cin >> SPI_MSG_LEN;

				// Allocate Data Array
				SPI_DATA_ARRAY = (unsigned int*) malloc(SPI_MSG_LEN * sizeof(unsigned int));

				if(SPIReadMultipleTest(DHCOM_SPIEnum(SPI_DEV - 1), DHCOM_SPIModeEnum(SPI_DEV_MODE), SPI_MSG_LEN, SPI_DATA_ARRAY) == true)
				{
					cStatus = "Success";
				}
				else
				{
					cStatus = "Failed";
				}

				cout << "SPI Read Multiple Test - Data: " << SPI_DATA_ARRAY << " is " << cStatus << "!!!" << endl;
				cout << "------------------------------------" << endl;

				delete(SPI_DATA_ARRAY);
			}
			break;
		case 5:
			{
				cout << endl;
				cout << "SPI Write NOR Flash Test" << endl;
				cout << "------------------------------------" << endl;
				cout << "Enter SPI Device (1,2): ";
				cin >> SPI_DEV;
				cout << "Enter SPI Mode (0..3): ";
				cin >> SPI_DEV_MODE;
				cout << "Enter SPI Register Address (e.g. 0 = 0x00): ";
				cin >> SPI_REG_ADDR;
				cout << "Enter SPI Data Len (e.g. 1): ";
				cin >> SPI_MSG_LEN;

				// Allocate Data Array
				SPI_DATA_ARRAY = (unsigned int*) malloc(SPI_MSG_LEN * sizeof(unsigned int));

				for(int i = 0; i < SPI_MSG_LEN; i++)
				{
					cout << "Enter SPI Data[" << i << "] (e.g. 0xFFFF): ";
					cin >> std::hex >> SPI_DATA_ARRAY[i];
				}

				if(SPIWriteFlash(DHCOM_SPIEnum(SPI_DEV - 1), DHCOM_SPIModeEnum(SPI_DEV_MODE), SPI_REG_ADDR, SPI_MSG_LEN, SPI_DATA_ARRAY) == true)
				{
					cStatus = "Success";
				}
				else
				{
					cStatus = "Failed";
				}

				cout << "SPI Write NOR Flash Test to " << SPI_REG_ADDR << " is " << cStatus << "!!!" << endl;
				cout << "------------------------------------" << endl;

				delete(SPI_DATA_ARRAY);
			}
			break;
		case 6:
			{
				cout << endl;
				cout << "SPI Read NOR Flash Test" << endl;
				cout << "------------------------------------" << endl;
				cout << "Enter SPI Device (1,2): ";
				cin >> SPI_DEV;
				cout << "Enter SPI Mode (0..3): ";
				cin >> SPI_DEV_MODE;
				cout << "Enter SPI Register Address (e.g. 0 = 0x00): ";
				cin >> std::hex >> SPI_REG_ADDR;
				cout << "Enter SPI Data Len (e.g. 1): ";
				cin >> SPI_MSG_LEN;

				// Allocate Data Array
				SPI_DATA_ARRAY = (unsigned int*) malloc(SPI_MSG_LEN * sizeof(unsigned int));

				if(SPIReadFlash(DHCOM_SPIEnum(SPI_DEV - 1), DHCOM_SPIModeEnum(SPI_DEV_MODE), SPI_REG_ADDR, SPI_MSG_LEN, SPI_DATA_ARRAY) == true)
				{
					cStatus = "Success";
				}
				else
				{
					cStatus = "Failed";
				}

				cout << "SPI Read NOR Flash Test from " << SPI_REG_ADDR << " with Register Value:" << SPI_DATA_ARRAY << " is " << cStatus << "!!!" << endl;
				cout << "------------------------------------" << endl;

				delete(SPI_DATA_ARRAY);
			}
			break;
		case 0:
			{
				delete(cStatus);
				return;
			}
			break;
		default:
			{

			}
			break;
		}
	}while(1);
}

// ****** SPI Functions ******

bool SPITest::SPIWriteTest(DHCOM_SPIEnum eSPIPort, DHCOM_SPIModeEnum eSPIMode, unsigned int *iData, unsigned int *iReverseData, bool bLoopback)
{
	if(SPIOpen(eSPIPort, eSPIMode, 16000000) == true)
	{
		if(SPIWrite(eSPIPort, iData, iReverseData, bLoopback) == true)
		{
			SPIClose(eSPIPort);
			return true;
		}

		SPIClose(eSPIPort);
	}

	return false;
}

bool SPITest::SPIReadTest(DHCOM_SPIEnum eSPIPort, DHCOM_SPIModeEnum eSPIMode, unsigned int *iData)
{
	if(SPIOpen(eSPIPort, eSPIMode, 16000000) == true)
	{
		if(SPIRead(eSPIPort, iData) == true)
		{
			SPIClose(eSPIPort);
			return true;
		}

		SPIClose(eSPIPort);
	}

	return false;
}

bool SPITest::SPIWriteMultipleTest(DHCOM_SPIEnum eSPIPort, DHCOM_SPIModeEnum eSPIMode, unsigned int iWCount, unsigned int *iData)
{
	if(SPIOpen(eSPIPort, eSPIMode, 16000000) == true)
	{
		if(SPIWriteMultiple(eSPIPort, iData, iWCount) == true)
		{
			SPIClose(eSPIPort);
			return true;
		}

		SPIClose(eSPIPort);
	}

	return false;
}

bool SPITest::SPIReadMultipleTest(DHCOM_SPIEnum eSPIPort, DHCOM_SPIModeEnum eSPIMode, unsigned int iWCount, unsigned int *iData)
{
	if(SPIOpen(eSPIPort, eSPIMode, 16000000) == true)
	{
		if(SPIReadMultiple(eSPIPort, iWCount, iData) == true)
		{
			SPIClose(eSPIPort);
			return true;
		}

		SPIClose(eSPIPort);
	}

	return false;
}

bool SPITest::SPIWriteFlash(DHCOM_SPIEnum eSPIPort, DHCOM_SPIModeEnum eSPIMode, unsigned int iRegAddr, unsigned int iWCount, unsigned int *iData)
{
	if(SPIOpen(eSPIPort, eSPIMode, 16000000) == true)
	{
		if(SPIWriteNORFlash(eSPIPort, iRegAddr, iData, iWCount) == true)
		{
			SPIClose(eSPIPort);
			return true;
		}

		SPIClose(eSPIPort);
	}

	return false;
}

bool SPITest::SPIReadFlash(DHCOM_SPIEnum eSPIPort, DHCOM_SPIModeEnum eSPIMode, unsigned int iRegAddr, unsigned int iWCount, unsigned int *iData)
{
	if(SPIOpen(eSPIPort, eSPIMode, 16000000) == true)
	{
		if(SPIReadNORFlash(eSPIPort, iRegAddr, (int)iWCount, iData) == true)
		{
			SPIClose(eSPIPort);
			return true;
		}

		SPIClose(eSPIPort);
	}

	return false;
}